@@ -386,9 +386,8 @@ class Serializer {
     if (!primaryResource && isEmpty(toInclude)) {
       return json;
     } else if (primaryResource) {
-      let [resourceHash, newIncludes] = this.getHashForPrimaryResource(
-        primaryResource
-      );
+      let [resourceHash, newIncludes] =
+        this.getHashForPrimaryResource(primaryResource);
       let newDidSerialize = this.isCollection(primaryResource)
         ? primaryResource.models
         : [primaryResource];
@@ -401,9 +400,8 @@ class Serializer {
       );
     } else {
       let nextIncludedResource = toInclude.shift();
-      let [resourceHash, newIncludes] = this.getHashForIncludedResource(
-        nextIncludedResource
-      );
+      let [resourceHash, newIncludes] =
+        this.getHashForIncludedResource(nextIncludedResource);
 
       let newToInclude = newIncludes
         .filter((resource) => {
@@ -412,9 +410,10 @@ class Serializer {
             .includes(resource.toString());
         })
         .concat(toInclude);
-      let newDidSerialize = (this.isCollection(nextIncludedResource)
-        ? nextIncludedResource.models
-        : [nextIncludedResource]
+      let newDidSerialize = (
+        this.isCollection(nextIncludedResource)
+          ? nextIncludedResource.models
+          : [nextIncludedResource]
       ).concat(didSerialize);
       let newJson = this.mergePayloads(json, resourceHash);
 
