@@ -472,9 +472,8 @@ class JSONAPISerializer extends Serializer {
             relationshipData = null;
           }
 
-          graph.data[graphKey].relationships[
-            graphRelationshipKey
-          ] = relationshipData;
+          graph.data[graphKey].relationships[graphRelationshipKey] =
+            relationshipData;
 
           if (relationship) {
             this._addResourceToRequestedIncludesGraph(
